import React from 'react';
export declare type ContinuousScaleConfigs = {
    scale: 'linear';
    domain?: number[];
} | {
    scale: 'square';
    domain?: number[];
} | {
    scale: 'log';
    domain?: number[];
    base?: number;
} | {
    scale: 'pow';
    domain?: number[];
    exponent?: number;
} | {
    scale: 'date';
    domain?: Date[];
};
export declare type CategoricalScaleConfigs = {
    scale: 'band';
    domain?: string[];
    padding?: number;
    paddingInner?: number;
    paddingOuter?: number;
} | {
    scale: 'point';
    domain?: string[];
    padding?: number;
};
export declare type OrdinalScaleConfigs = {
    scale: 'ordinal';
    domain: string[];
    range: string[];
};
export declare type ScaleConfigs = ContinuousScaleConfigs | CategoricalScaleConfigs;
export declare type CartesianScaleType = ScaleConfigs['scale'];
export declare type CartesianScaleOptions<T extends CartesianScaleType> = Extract<ScaleConfigs, {
    scale: T;
}>;
export declare type CartesianScaleDomain<T extends CartesianScaleType> = NonNullable<CartesianScaleOptions<T>['domain']>;
declare type Callback<Event> = (e: Event) => void;
export declare type CartesianGestureCallback<X extends CartesianScaleType, Y extends CartesianScaleType> = Callback<{
    xDomain: CartesianScaleDomain<X>;
    yDomain: CartesianScaleDomain<Y>;
}>;
interface CartesianProps<X extends CartesianScaleType, Y extends CartesianScaleType> {
    x?: CartesianScaleOptions<X>;
    y?: CartesianScaleOptions<Y>;
    /** @deprecated */
    color?: OrdinalScaleConfigs;
    nice?: 'x' | 'y' | boolean;
    children?: React.ReactNode;
    interactive?: boolean;
}
export declare function Cartesian<X extends CartesianScaleType = 'linear', Y extends CartesianScaleType = 'linear'>({ x, y, color, children, nice, interactive, }: CartesianProps<X, Y>): JSX.Element;
export {};
