import { EventHandler, SyntheticEvent } from 'react';
import { EventsHandlers, SvgAttributes } from './types.svg';
export declare const tuple: <Args extends any[]>(...args: Args) => Args;
export declare const strTuple: <Args extends string[]>(...args: Args) => Args;
export declare const isUndefinedTuple: (x: [any, any]) => x is [undefined, undefined];
export declare const isNotNil: <T>(t: T | null | undefined) => t is T;
/**
 * Type used to prettify complex type
 */
declare type Id<T> = {
    [K in keyof T]: T[K];
};
declare type Get<Key, From> = Key extends keyof From ? From[Key] : never;
/**
 * Picks undefined keys
 */
declare type UndefinedProperties<T> = {
    [P in keyof T]-?: undefined extends T[P] ? P : never;
}[keyof T];
/**
 * Turn undefined keys as optional
 */
declare type ToOptional<T> = Id<Partial<Pick<T, UndefinedProperties<T>>> & Omit<T, UndefinedProperties<T>>>;
/**
 * Merge objects
 */
export declare type Merge<A, B> = ToOptional<{
    [K in keyof A | keyof B]: Get<K, A> | Get<K, B>;
}>;
export declare type Component<Props> = (props: Props) => JSX.Element;
export declare type SvgAttributesGetters<T> = {
    [K in keyof SvgAttributes]?: Getter<T, NonNullable<SvgAttributes[K]>>;
};
declare type NativeEvent<T> = T extends EventHandler<infer V> ? V extends SyntheticEvent<any, infer E> ? E : never : never;
export declare type NativeEventHandlers<T> = {
    [Event in keyof EventsHandlers]?: (event: NativeEvent<Required<EventsHandlers>[Event]>, datum: T) => void;
};
export declare type Iteratee<T, U> = (t: T, i: number) => U;
export declare type DefaultedIteratee<T, U> = U | Iteratee<T, U>;
export declare function wrapIteratee<A, B, C>(iteratee: Iteratee<B, C>, map: Accessor<A, B>): Iteratee<A, C>;
export interface CommonAnimationProps {
    delay: number;
    duration: number;
    easing: (time: number) => number;
}
export interface AnimationIteratees<T> {
    delay?: DefaultedIteratee<T, number>;
    duration?: DefaultedIteratee<T, number>;
    easing?: (time: number) => number;
}
export interface AnimationProps<T> extends AnimationIteratees<T> {
    dataKey?: KeyAccessor<T>;
    enter?: SvgAttributes;
}
export declare type Accessor<Datum, Value> = (t: Datum) => Value;
export declare type Getter<Datum, Value> = Value | Accessor<Datum, Value>;
export declare function isAccessor<T, V>(getter: Getter<T, V>): getter is Accessor<T, V>;
export declare function toAccessor<T, V>(getter: Getter<T, V>): Accessor<T, V>;
export declare type DataValue = string | number | Date;
export declare type DataAccessor<T> = Accessor<T, DataValue>;
export declare type DataGetter<T> = Getter<T, DataValue>;
export declare type KeyAccessor<T> = (t: T, index: number) => DataValue;
export declare type Filter<T> = (value: T, index: number, array: T[]) => boolean;
export declare type ScaleContinuous = ((value: number | Date) => number) & {
    invert: (value: number) => number | Date;
    nice(): ScaleContinuous;
    copy(): ScaleContinuous;
    ticks(count?: number): number[] | Date[];
    range(): number[];
};
export declare type ScaleCategorical = ((value: string) => number) & {
    copy(): ScaleCategorical;
    bandwidth(): number;
    domain(): string[];
    range(): number[];
};
export declare type ScaleOrdinal = (value: string) => string;
export declare const isScaleContinuous: (v: any) => v is ScaleContinuous;
export declare const isScaleCategorical: (v: any) => v is ScaleCategorical;
export declare type CartesianScale = ScaleContinuous | ScaleCategorical;
export {};
